"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.call_dest = exports.call_src = exports.test_dest = exports.test_src = void 0;
const node_fetch_1 = __importDefault(require("node-fetch"));
function urlArgsToString(urlArgs) {
    let keys = Object.keys(urlArgs);
    if (keys.length == 0) {
        return '';
    }
    let s = '?';
    keys.forEach(key => {
        let value = urlArgs[key];
        let value_str = value;
        s += '&' + key + '=' + value_str;
    });
    return s;
}
function test_src(payload, sam_port, sam_host) {
    if (typeof payload.body !== 'string') {
        payload.headers['Content-Type'] = 'application/json';
    }
    return node_fetch_1.default(`http://${sam_host || '127.0.0.1'}:${sam_port}/function${urlArgsToString(payload.queryParameters)}`, {
        method: 'post',
        body: typeof payload.body == 'string' ? payload.body : JSON.stringify(payload.body),
        headers: payload.headers
    }).then(r => r.json());
}
exports.test_src = test_src;
async function test_dest(payload, sam_port, sam_host) {
    return node_fetch_1.default(`http://${sam_host || '127.0.0.1'}:${sam_port}/function`, {
        method: 'post',
        body: JSON.stringify(payload),
        headers: {
            'Content-Type': 'application/json'
        }
    });
}
exports.test_dest = test_dest;
async function call_src(payload, onRequest, output) {
    output.tracks = [];
    output.identifies = [];
    output.groups = [];
    output.aliases = [];
    output.screens = [];
    output.pages = [];
    global.Segment = {
        identify(i) {
            output.identifies.push(i);
        },
        track(t) {
            output.tracks.push(t);
        },
        group(t) {
            output.groups.push(t);
        },
        alias(t) {
            output.aliases.push(t);
        },
        page(t) {
            output.pages.push(t);
        },
        screen(t) {
            output.screens.push(t);
        }
    };
    let body = payload.body;
    const settings = {};
    await onRequest({
        json() {
            return JSON.stringify(body);
        },
        text() {
            return body;
        },
        headers: new Headers(payload.headers),
        url: new URL(payload.path + urlArgsToString(payload.queryStringParameters))
    }, settings);
}
exports.call_src = call_src;
async function call_dest(_event, fns) {
    let event = JSON.parse(_event.body);
    const settings = {};
    switch (event.type) {
        case 'alias': {
            if (fns.onAlias) {
                fns.onAlias(event, settings);
            }
            break;
        }
        case 'identify': {
            if (fns.onIdentify) {
                fns.onIdentify(event, settings);
            }
            break;
        }
        case 'page': {
            if (fns.onPage) {
                fns.onPage(event, settings);
            }
            break;
        }
        case 'screen': {
            if (fns.onScreen) {
                fns.onScreen(event, settings);
            }
            break;
        }
        case 'track': {
            if (fns.onTrack) {
                fns.onTrack(event, settings);
            }
            break;
        }
        case 'group': {
            if (fns.onGroup) {
                fns.onGroup(event, settings);
            }
            break;
        }
    }
}
exports.call_dest = call_dest;
//# sourceMappingURL=index.js.map