import { transpile, ScriptTarget } from 'typescript'
import { listFunctions, updateFunction, createFunction } from 'segment-typescript-api/cjs/config_api'

import * as fs from 'fs'
// import * as path from 'path'
import { Settings, get_source_file } from './settings'

// import { updateFunction, createFunction } from './gateway/api'
// import { getWorkspaceId as _getWorkspaceId, get_function_ids } from './gateway/request'
// import { FunctionType } from './gateway/api'
// import { Settings } from 'http2'
// import { Settings } from './settings'

// export async function deploy_functions(settings: Settings, url: string, debug: boolean) {
//   if (settings.)
//   // await Promise.all([
//   //   settings.protocol.src_fn_name ? deploy_source(path.join(process.cwd(), 'src'), true, url, settings.protocol.src_fn_name, settings.protocol.work_id, settings.server.access_token).then(() => {
//   //     console.log('Deployed source ' + settings.protocol.src_fn_name)
//   //   }) : Promise.resolve(),
//   //   settings.protocol.dest_fn_name ? deploy_destination(path.join(process.cwd(), 'src'), true, url, settings.protocol.dest_fn_name, settings.protocol.work_id, settings.server.access_token).then(() => {
//   //     console.log('Deployed destination ' + settings.protocol.dest_fn_name)
//   //   }) : Promise.resolve()
//   // ])
//   if (settings.protocol.src_fn_name) {
//     let src = path.join(process.cwd(), 'src')
//     src = fs.existsSync(src) ? src : process.cwd()
//     await deploy_source(src, debug, url, settings.protocol.src_fn_name, settings.protocol.work_slug, settings.server.access_token)
//     console.log('Deployed source ' + settings.protocol.src_fn_name)
//   }

//   if (settings.protocol.dest_fn_name) {
//     let src = path.join(process.cwd(), 'src')
//     src = fs.existsSync(src) ? src : process.cwd()
//     await deploy_destination(src, debug, url, settings.protocol.dest_fn_name, settings.protocol.work_slug, settings.server.access_token)
//     console.log('Deployed destination ' + settings.protocol.dest_fn_name)
//   }
// }

export async function get_js(settings: Settings) {
  let fn_path = get_source_file(settings)
  if (settings.language == 'typescript') {
    let ts = await fs.promises.readFile(fn_path, 'utf8')
    return transpile(ts, {
      target: ScriptTarget.ES2017
    })
  } else {
    return await fs.promises.readFile(fn_path, 'utf8')
  }

}

export function prepare_source_code(js: string) {
  return js
    .replace(/Object\.defineProperty\(exports\, \"__esModule\"\, \{ value\: true \}\)\;/, '')
    .replace(/exports\.onRequest \= void 0\;/, '')
    .replace(/exports[\s]*\.onRequest[\s]*\=[\s]*onRequest[;]?/, '')
}

export function prepare_destination_code(js: string) {
  return js
    .replace(/Object\.defineProperty\(exports\, \"__esModule\"\, \{ value\: true \}\)\;/, '')
    .replace(/exports\.onTrack \= void 0\;/, '')
    .replace(/exports[\s]*\.onTrack[\s]*\=[\s]*onTrack[;]?/, '')
    .replace(/exports\.onIdentify \= void 0\;/, '')
    .replace(/exports[\s]*\.onIdentify[\s]*\=[\s]*onIdentify[;]?/, '')
    .replace(/exports\.onAlias \= void 0\;/, '')
    .replace(/exports[\s]*\.onAlias[\s]*\=[\s]*onAlias[;]?/, '')
    .replace(/exports\.onGroup \= void 0\;/, '')
    .replace(/exports[\s]*\.onGroup[\s]*\=[\s]*onGroup[;]?/, '')
    .replace(/exports\.onPage \= void 0\;/, '')
    .replace(/exports[\s]*\.onPage[\s]*\=[\s]*onPage[;]?/, '')
    .replace(/exports\.onScreen \= void 0\;/, '')
    .replace(/exports[\s]*\.onScreen[\s]*\=[\s]*onScreen[;]?/, '')
}

export async function deploy_source(settings: Settings, access_token?: string, debug_url?: string) {//fn_path: string, debug: boolean, url: string, fn_name: string, work_id: string, access_token: string) {
  if (!settings.work_id || !(settings.access_token || access_token)) {
    throw 'Cannot deploy without workspace ID and access token'
  }
  let js = ''

  if (debug_url) {
    if (debug_url.charAt(debug_url.length - 1)) {
      debug_url = debug_url + '/'
    }
    js = `
function urlArgsToString(urlArgs) {
  let keys = Object.keys(urlArgs);
  if (keys.length == 0) {
    return ''
  }
  let s = '?'
  keys.forEach(key => {
    let value = urlArgs[key]
    let value_str = value
    s += '&' + key + '=' + value_str
  })
  return s
}
    
async function onRequest(request, settings) {
  let payload = await request.json()
  if (typeof payload.body !== 'string') {
    payload.headers['Content-Type'] = 'application/json'
  }
  return fetch('${debug_url}' + urlArgsToString(payload.queryParameters), {
    method: 'post',
    body: typeof payload.body == 'string' ? payload.body : JSON.stringify(payload.body),
    headers: payload.headers
  })
}`
  } else {
    let js = await get_js(settings)

    js = prepare_source_code(js)



    // // let ts_path = path.join(src_fn_path, 'src_fn.ts')
    // // let js_path = path.join(src_fn_path, 'src_fn.js')
    // if (fs.existsSync(ts_path)) {
    //   let sourceCode = await fs.promises.readFile(ts_path, 'utf8')

    //   js = transpile(sourceCode, {
    //     target: ScriptTarget.ES2017
    //   })

    //   js = js.replace(/export async/g, 'async').replace("import 'segment-typescript-definitions/common'", '').replace("import 'segment-typescript-definitions/custom-source'", '')
    // } else if (fs.existsSync(js_path)) {
    //   let sourceCode = await fs.promises.readFile(js_path, 'utf8')
    //   js = sourceCode
    //     .replace(/exports.onRequest[\s]*=[\s]*onRequest/, '')
    // } else {
    //   throw 'Cannot find src_fn file (.ts or .js)'
    // }
  }

  await build_function(settings.fn_name, true, settings.work_id, js, (access_token || settings.access_token) as string)
}

export async function deploy_destination(settings: Settings, access_token?: string, debug_url?: string) {//fn_path: string, debug: boolean, url: string, fn_name: string, work_id: string, access_token: string) {
  if (!settings.work_id || !(settings.access_token || access_token)) {
    throw 'Cannot deploy without workspace ID and access token'
  }
  let js = ''

  if (debug_url) {
    js = `async function onTrack(event, settings) {
  await fetch('${debug_url}', {
    method: 'post',
    headers: {
      'Content-Type': 'application/json'
    },
    body: JSON.stringify({
      event, settings
    })
  })
}
async function onIdentify(event, settings) {
  await fetch('${debug_url}', {
    method: 'post',
    headers: {
      'Content-Type': 'application/json'
    },
    body: JSON.stringify({
      event, settings
    })
  })
}
async function onAlias(event, settings) {
  await fetch('${debug_url}', {
    method: 'post',
    headers: {
      'Content-Type': 'application/json'
    },
    body: JSON.stringify({
      event, settings
    })
  })
}
async function onPage(event, settings) {
  await fetch('${debug_url}', {
    method: 'post',
    headers: {
      'Content-Type': 'application/json'
    },
    body: JSON.stringify({
      event, settings
    })
  })
}
async function onScreen(event, settings) {
  await fetch('${debug_url}', {
    method: 'post',
    headers: {
      'Content-Type': 'application/json'
    },
    body: JSON.stringify({
      event, settings
    })
  })
}
async function onGroup(event, settings) {
  await fetch('${debug_url}', {
    method: 'post',
    headers: {
      'Content-Type': 'application/json'
    },
    body: JSON.stringify({
      event, settings
    })
  })
}`
  } else {
    let js = await get_js(settings)
    // let js = await fs.promises.readFile(fn_path, 'utf8')
    js = prepare_destination_code(js)

    // let ts_path = path.join(dest_fn_path, 'dest_fn.ts')
    // let js_path = path.join(dest_fn_path, 'dest_fn.js')

    // if (fs.existsSync(ts_path)) {
    //   let sourceCode = await fs.promises.readFile(ts_path, 'utf8')

    //   js = transpile(sourceCode, {
    //     target: ScriptTarget.ES2017
    //   })

    //   js = js.replace(/export async/g, 'async').replace("import 'segment-typescript-definitions/common'", '').replace("import 'segment-typescript-definitions/custom-destination'", '')
    // } else {
    //   let sourceCode = await fs.promises.readFile(js_path, 'utf8')
    //   js = sourceCode
    //     .replace(/exports.onScreen[\s]*=[\s]*onScreen/, '')
    //     .replace(/exports.onGroup[\s]*=[\s]*onGroup/, '')
    //     .replace(/exports.onTrack[\s]*=[\s]*onTrack/, '')
    //     .replace(/exports.onAlias[\s]*=[\s]*onAlias/, '')
    //     .replace(/exports.onIdentify[\s]*=[\s]*onIdentify/, '')
    //     .replace(/exports.onPage[\s]*=[\s]*onPage/, '')
    // }
  }

  await build_function(settings.fn_name, false, settings.work_id, js, (access_token || settings.access_token) as string)
}

// async function try_build<T>(build: () => Promise<T>, retries?: number, retry_backoff?: number) {
//   if (retries) {
//     let backoff = retry_backoff || 500
//     let last_error: any
//     for (let i = 0; i < retries; i++) {
//       try {
//         let v = await build()
//         return v
//       } catch (e) {
//         last_error = e
//         console.error('Build failed with error: ' + e)
//       }
//       await new Promise(resolve => setTimeout(resolve, backoff))
//       backoff *= 2
//     }
//     throw last_error
//   } else {
//     return await build()
//   }
// }

export async function build_function(fn_name: string, is_src: boolean, work_id: string, code: string, access_token: string, dont_overwrite?: true): Promise<string | false> {
  let fns = await listFunctions(access_token, work_id, { type: is_src ? 'SOURCE' : 'DESTINATION', page_size: 100 })
  let fn = fns.functions ? fns.functions.find(fn => fn.display_name == fn_name) : undefined

  if (fn) {
    if (dont_overwrite) {
      return false
      // throw `Function with name '${fn_name}' already exists`
    }
    let _fn = fn
    let r = await updateFunction(access_token, work_id, _fn.id, {
      function: {
        code,
        buildpack: 'boreal'
      },
      update_mask: {
        paths: ['function.code', 'function.buildpack']
      }
    })
    return r.id
  } else {
    let r = await createFunction(access_token, work_id, is_src ? 'SOURCE' : 'DESTINATION', {
      code,
      buildpack: 'boreal',
      display_name: fn_name,
    })
    return r.id
  }
}