import { exec, spawn } from 'child_process'
import { Settings } from './settings'

export function nodemon_installed(): Promise<boolean> {
  return new Promise((resolve, reject) => {
    exec(`nodemon --version`, (error, std_out) => {
      if (error) {
        reject(error)
      } else {
        resolve(std_out.substring(0, 1).match(/\d/) !== null)
      }
    })
  })
}

export async function get_exec(settings: Settings, is_debug: boolean) {
  let e = `sam local start-api --port ${settings.sam_port}`
  if (is_debug) {
    e = `${e} --debug-port ${settings.debug_port}`
  }
  if (settings.language == 'typescript') {
    e = `tsc -p .;${e}`
  }

  let is_nodemon = await nodemon_installed()
  if (is_nodemon) {
    e = `nodemon --watch src -e ts --exec "${e}"`
  }

  return e
}

export async function run_exec(settings: Settings, is_debug: boolean) {
  let e = await get_exec(settings, is_debug)
  let s = spawn(e, { shell: true, stdio: 'inherit' })
  // s.stdout.on('data', data => {
  //   console.log(data.toString())
  // })
  // s.stderr.on('data', data => {
  //   console.error(data.toString())
  // })
  return s
}